@@ -42,7 +42,10 @@ def convert_int_list(strlist):
     intlist = []
     for s1 in strlist:
         if s1 != '':
-            intlist.append(int(s1))
+            try:
+                intlist.append(int(s1))
+            except ValueError:
+                pass
     return intlist
 
 def comma(value):
@@ -83,7 +86,6 @@ def fileSave(path, cont, opt, gzip_flag="n"):
         f.close
 
 
-
 def fileOpen(path):
     f = open(path, "r")
     return f.read()
@@ -138,4 +140,10 @@ def load_json(jsonfile):
     ds = ""
     with open(jsonfile) as jfp:
         ds = json.load(jfp)
-    return ds
\ No newline at end of file
+    return ds
+
+def renderTemplate(templatefile, outfile, data={}):
+    cont = fileOpen(getTemplatePath(templatefile))
+    for k1 in data.keys():
+        cont = cont.replace('##' + k1+ '##', data[k1])
+    fileSave(outfile, cont, 'w')
\ No newline at end of file
